#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
import numpy as np


from pyrfu import mms, pyrf, models, dispersion

__author__ = "Louis Richard"
__email__ = "louisr@irfu.se"
__copyright__ = "Copyright 2020-2021"
__license__ = "MIT"
__version__ = "2.3.7"
__status__ = "Prototype"
